% % function FMCW_Range_Doppler()
clear all
close all
clc
          
%%%%Signal Parameter
fTast = 5e4; %Sample Freq
fMaxDist = 40;
%%%% Radar Parameters
fCenter = 24.125e9;
sweepDur = 5e-3;
sweepBwd = 250e6;
snrdB = 20;
fC = 3e8;
fLambda = fC / fCenter;
fOmegaStart = 2*pi* (fCenter -sweepBwd/2);
npkt = round(sweepDur*fTast+1);
I=45; % number of measures 
sweepDurm = 5e-3; % measurement period (T0)
finalmat = zeros (I,npkt);
T=1;

%%%Antennas Position
a = 0.014; %m
AntPos =  [-a/2,0,0; %rx1
           a/2,0,0;  %rx2
           0,0,0;    %tx 
              ];

% Targets Trajectories
% Target 1
A1=1;
v1.x =0; 
v1.y = 0.1*sweepDurm; % speed in m/T0
v1.z = 0;
thet1=23*pi/180;    % angle in rad
r1=4;               % range in m
yy1=cos(thet1)*r1-(0.255*0.03);
xx1=sin(thet1)*r1;
Pos = [xx1,yy1,0];
Traj1 = traj_linear (I, v1, T, Pos);
% Target 2
A2=0.7;
v2.x = 0;
v2.y = 0*sweepDurm; % speed in m/T0
v2.z = 0;
thet2=-21*pi/180;    % angle in rad
r2=10;              % range in m
yy2=cos(thet2)*r2-(0.255*0.01);
xx2=sin(thet2)*r2;
Pos = [xx2,yy2,0];
Traj2 = traj_linear (I, v2, T, Pos);
% Target 3
A3=0.4;
v3.x = 0; %(m/T)
v3.y = -0.2*sweepDurm; % speed in m/T0
v3.z = 0;
thet3=19*pi/180; % angle in rad
r3=17; % range in m
yy3=cos(thet3)*r3-(0.02*0.225);
xx3=sin(thet3)*r3;
Pos = [xx3,yy3,0];
Traj3 = traj_linear (I, v3, T, Pos);
% Target 4
A4=0.5;
v4.x = 0; %(m/T)
v4.y = 0.33*sweepDurm; % speed in m/T0
v4.z = 0;
thet4=-7*pi/180; % angle in rad
r4=25; % range in m
yy4=cos(thet4)*r4;
xx4=sin(thet4)*r4;
Pos = [xx4,yy4,0];
Traj4 = traj_linear (I, v4, T, Pos);


for iCnt = 1: I

%%%%%%%%%%%%%%%%%%Radar Signal

debug = 1;

% distance between targets to receivers position
TargetDist1 = target_dist(1,Traj1,Traj2,Traj3,Traj4,AntPos,iCnt);
TargetDist2 = target_dist(2,Traj1,Traj2,Traj3,Traj4,AntPos,iCnt);

Dist1= TargetDist1 ;
Dist2= TargetDist2 ;
TargetAmp = [A1 A2 A3 A4];

normAxis = (0:(npkt-1))/(npkt-1); % normal axis
timeAxis = normAxis*sweepDur; % time axis
% setup sweep axis
fStart = fCenter - sweepBwd/2;
SweepFreqUp =repmat((normAxis*sweepBwd)',1,length(TargetDist1)); 

% generate distance/time and amplitude matrix
TargetDistMat1 = repmat(TargetDist1, npkt,1); 
TargetDistMat2 = repmat(TargetDist2, npkt,1); 
tau1 = (TargetDistMat1/ fC); 
tau2 = (TargetDistMat2/ fC); 

TargetAmpMat = repmat(TargetAmp,npkt,1);

% calculate/simulate the signal 
 for Ziel_N = 1:size(TargetAmpMat,2)
    sigUp1(Ziel_N,:) = sum(TargetAmpMat(:,Ziel_N) .* exp(j*2*pi*(fStart.*tau1(:,Ziel_N) +  SweepFreqUp(:,Ziel_N).*tau1(:,Ziel_N))),2)';
 end
 sigUp1 = sum(sigUp1); % Beat signal antenna 1
 
 for Ziel_N = 1:size(TargetAmpMat,2)
    sigUp2(Ziel_N,:) = sum(TargetAmpMat(:,Ziel_N) .* exp(j*2*pi*(fStart.*tau2(:,Ziel_N) +  SweepFreqUp(:,Ziel_N).*tau2(:,Ziel_N))),2)';
 end
 sigUp2 = sum(sigUp2); % Beat signal antenna 2

 
% calculate white noise with snr 20dB
rand('seed',sum(10000*clock)); %double-precision value in interval[1/(2^31-1),1-1/(2^31-1)], see doc rand
noiseUp = rand(1,npkt)-0.5;
fgo = 2 * 0.45;
[sfB sfA] = cheby1(2, 1,  fgo);
noiseUp = filtfilt(sfB, sfA, noiseUp);        
noiseUp = noiseUp - mean(noiseUp);

% Normalized noise
UMaxEff = 1 / sqrt(2) * max(abs(TargetAmp)) ;  % Definition of SNR 
UrEff   = sqrt(mean(noiseUp .* conj(noiseUp)));
aNoiseL = UMaxEff / UrEff / (10 ^ (snrdB/ 20));
noiseUp = aNoiseL * noiseUp;
sigUp1 = real(sigUp1 + noiseUp); 
sigUp2 = real(sigUp2 + noiseUp); 

finalmat1 ((iCnt),:) = sigUp1;
finalmat2 ((iCnt),:) = sigUp2;
end


% calculate the echo profil
wfkt = hamming(I) * hamming(npkt)' ; % 2D hamming window
% ipolN = (2^nextpow2(npkt * 8)); 
ipolN = 3000;
specUp1 = fft2(finalmat1.*wfkt,ipolN,ipolN) *  2  / npkt / mean(mean(wfkt)) ; % Echo Spectra
specUp2 = fft2(finalmat2.*wfkt,ipolN,ipolN) *  2  / npkt / mean(mean(wfkt)) ; % Echo Spectra

[dopCells rCells] = size(specUp1);

specUp1 = fftshift (specUp1);
specUp1 = specUp1 (:,ipolN/2:ipolN);
specUp2 = fftshift (specUp2);
specUp2 = specUp2 (:,ipolN/2:ipolN);


fAxis = (0:((rCells/2)-1))  / rCells*fTast ; % frecuency axis
rAxis = fAxis * fC * sweepDur / 2 / sweepBwd;
nMax=min(find(rAxis>fMaxDist));
rAxis = rAxis(1:nMax);

specUp1 = specUp1(:,1:nMax);
specUp2 = specUp2(:,1:nMax);

T0 = sweepDurm;
fTastdop = I/(T0*I);
dopAxis = (([1:dopCells]-dopCells/2) / dopCells*fTastdop);
dopAxis = (dopAxis*fC)/(2*fCenter);

res1 =abs(specUp1)/max(max(abs(specUp1)));
res2 =abs(specUp2)/max(max(abs(specUp2)));

figure(1),subplot(211),imagesc(rAxis,dopAxis,-1*res1);
set(gca, 'yDir','normal','xgrid','on','ygrid','on');
 colormap(hot);
        grid on;
        ylabel ('speed in m/s');
        xlabel ('pseudo-range in m')
                title('spectrum antenna 1')
subplot(212),imagesc(rAxis,dopAxis,-1*res2);
set(gca, 'yDir','normal','xgrid','on','ygrid','on');
 colormap(hot);
        grid on;
        ylabel ('speed in m/s');
        xlabel ('pseudo-range in m')
title('spectrum antenna 2')

specUpM = (specUp1) .* conj(specUp2);

res =abs(specUpM)/max(max(abs(specUpM)));
figure(2),imagesc(rAxis,dopAxis,-1*res);
 set(gca, 'yDir','normal','xgrid','on','ygrid','on');
 colormap(hot);
        grid on;
        ylabel ('speed in m/s');
        xlabel ('pseudo-range in m')
        title('Multiplied Spectrum')

[pos_max,nro_max,int_max]= max_matrix(res) ;

cmap=flipud(colormap('Autumn'));
color = round(64.*((int_max)/(max(int_max)))); % colormap is composed by 64 colors

specUp_angle = angle(specUpM);
for i=1:nro_max
    delta_phase(i) = specUp_angle(pos_max(i,1),pos_max(i,2));
end

beta_rad = (1)*asin((delta_phase*fLambda)/(2*pi*a));
beta_grad = round_n((180*beta_rad)/pi,3)

% limit (beta_rad must be less than limit to indicate a correct value of angle)
beta_rad_limit = asin(fLambda/(2*a));
beta_grad_limit = (180*beta_rad_limit)/pi

x = linspace (0,40,400);
limit_line_p = x .* (cos(beta_rad_limit+(pi/2))/sin(beta_rad_limit+(pi/2)));
limit_line_m = x .* (cos(-beta_rad_limit+(pi/2))/sin(-beta_rad_limit+(pi/2)));

dist_meas = round_n(rAxis (pos_max(:,2)),5)

% calculate position of each target with range and angle calculated.
    xm = dist_meas  .* sin(beta_rad);
    ym = dist_meas  .* cos(beta_rad);
    zm = 0; 
    
for d=1:nro_max
    figure(3),
    title('Range in m; Angle in ; Speed in m/s'),
    range='  R=';
    ran=num2str(round_n(dist_meas(d),5));
    angl='; A=';
    ang=num2str(beta_grad(d));
    speed='; Sp=';
    spe=num2str(round_n(dopAxis(pos_max(d,1)),2));
    a=strcat(range,ran,angl,ang,speed,spe);
    text(xm(d),ym(d),texlabel(a),'BackgroundColor',[.9 .9 .7]),
    hold on
    
    grosse = 5.*((int_max(d)))/(max(int_max));
    figure(4),plot(AntPos(1,1),AntPos(1,2),'+b'),hold on
    plot(AntPos(2,1),AntPos(2,2),'+b'),hold on
    colormap(cmap)
    hcb=colorbar('location','EastOutside','YTickLabel',...
         {'20%','30%','45%','65%','75%','90%'})
     set(hcb,'YTickMode','manual')
    plot(limit_line_p,x,'--k'),hold on
    plot(limit_line_m,x,'--k');
    xlabel('x in m'),ylabel('y in m')
     plot(xm(d),ym(d),'^','LineWidth',grosse,...
        'MarkerEdgeColor',[cmap(color(d),1) cmap(color(d),2) cmap(color(d),3)],...
        'MarkerFaceColor',[cmap(color(d),1) cmap(color(d),2) cmap(color(d),3)]),hold on   
end

figure(5),polar(0,40),hold on
polar(beta_rad,dist_meas,'bs'),title('Polar scheme')

    
        

        
